unit UnidadDatos;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, adshlp, ActiveDs_TLB, StdCtrls, ExtCtrls, ImgList, ComCtrls, activex;

type
  TformDatos = class(TForm)
    lsUsuarios: TTreeView;
    UserImages: TImageList;
    Panel1: TPanel;
    Label1: TLabel;
    txtDominio: TEdit;
    bCambiarDominio: TButton;
    Panel2: TPanel;
    bExportar: TButton;
    dlGuardar: TSaveDialog;
    bCerrar: TButton;
    GroupBox1: TGroupBox;
    lbGroups: TListBox;
    lbGroupPath: TListBox;
    Button2: TButton;
    Button3: TButton;
    bExportarTodos: TButton;
    GroupBox2: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    txtUsuario: TEdit;
    txtNombreCompleto: TEdit;
    txtDescripcion: TEdit;
    opCambioContraProx: TCheckBox;
    opNoCambioContra: TCheckBox;
    opContraExpira: TCheckBox;
    opCuentaDesactivada: TCheckBox;
    opCuentaBloqueada: TCheckBox;
    txtEmail: TEdit;
    Label5: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure insertarArbol (disp: IADs);
    procedure bCambiarDominioClick(Sender: TObject);
    procedure bExportarClick(Sender: TObject);
    procedure bCerrarClick(Sender: TObject);
    procedure lsUsuariosChange(Sender: TObject; Node: TTreeNode);
    procedure EnumGroups(usr: IADsUser);
    procedure bExportarTodosClick(Sender: TObject);
  private
    { Private declarations }
    dom : IADsContainer;
  public
    { Public declarations }
    function GetGroupType( grp : IADsGroup ) : String;
  end;

var
  formDatos: TformDatos;

implementation

uses UnidadMenuPrincipal;

{$R *.dfm}

function siNo (valor : Boolean) : string;
begin
  if valor then
    Result := 'S'
  else
    Result := 'No';
end;

function TformDatos.GetGroupType( grp : IAdsGroup) : string;
var
  groupType : OleVariant;
begin
  Result := 'Unknown';
  groupType := grp.Get('groupType');
  If (groupType = ADS_GROUP_TYPE_GLOBAL_GROUP) Then
    Result := 'Global Group'
  Else If (groupType = ADS_GROUP_TYPE_DOMAIN_LOCAL_GROUP) Then
      Result := 'Local Group'
end;


procedure TformDatos.EnumGroups(usr: IADsUser);
var
  Enum : IEnumVariant;
  grps : IAdsMembers;
  sGroupType : string;
  grp : IAdsGroup;
  varGroup : OleVariant;
  Temp : LongWord;
begin
  lbGroups.Items.Clear;
  lbGroupPath.Items.Clear;
  grps := usr.Groups;
  Enum := grps._NewEnum as IEnumVariant;
  if Enum <> nil then
  begin
    while (Enum.Next(1,varGroup, Temp) = S_OK) do
    begin
      grp := IDispatch(varGroup) as IAdsGroup;
      sGroupType := GetGroupType(grp);
      lbGroups.Items.Add(grp.Name + ' (' +  sGroupType + ')');
      lbGroupPath.Items.Add(grp.ADsPath);
      VariantClear(varGroup);
    end;
  end;
end;


procedure TformDatos.insertarArbol (disp: IADs);
begin
  lsUsuarios.Items.Add(nil, disp.Name);
end;


procedure TformDatos.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  formMenuPrincipal.Show;
end;

procedure TformDatos.FormShow(Sender: TObject);
var
  dominio : string;
begin
  if formMenuPrincipal.opDominio.Checked then
    dominio := formMenuPrincipal.txtDominio.Text
  else
    dominio := formMenuPrincipal.txtEquipo.Text;
  txtDominio.Text := dominio;
  if formMenuPrincipal.opConectarUsuario.Checked then
    ADsOpenObject('WinNT://' + dominio,
        formMenuPrincipal.txtUsuario.Text,
        formMenuPrincipal.txtContrasena.Text, 1, IADsContainer,
        dom)
  else
    ADsGetObject('WinNT://' + dominio, IADsContainer, dom);
  dom.Filter := VarArrayOf(['user']);
  lsUsuarios.Items.Clear;
  ADsEnumerateObjects(dom, insertarArbol);
end;

procedure TformDatos.bCambiarDominioClick(Sender: TObject);
begin
  formMenuPrincipal.Show;
end;

procedure TformDatos.bExportarClick(Sender: TObject);
begin
  if MessageDlg('Se exportarn slo los nombres de los usuarios del dominio a un ' +
      'fichero de texto plano. Desea continuar?',
      mtConfirmation, [mbyes, mbno], 0) = mryes then
  begin
    if dlGuardar.Execute then
      lsUsuarios.SaveToFile(dlGuardar.FileName);
  end;
end;

procedure TformDatos.bCerrarClick(Sender: TObject);
begin
  close;
end;

procedure TformDatos.lsUsuariosChange(Sender: TObject; Node: TTreeNode);
var
  usr : IAdsUser;
  flags : integer;
  dominio : string;
begin
  if Node = nil then
    Exit;
  if formMenuPrincipal.opDominio.Checked then
    dominio := formMenuPrincipal.txtDominio.Text
  else
    dominio := formMenuPrincipal.txtEquipo.Text;
  if formMenuPrincipal.opConectarUsuario.Checked then
    ADsOpenObject('WinNT://' + dominio + '/' +
        Node.Text, formMenuPrincipal.txtUsuario.Text,
        formMenuPrincipal.txtContrasena.Text, 1,
        IADsUser, usr)
  else
    ADsGetObject('WinNT://' + dominio + '/' + Node.Text, IADsUser, usr);

  Assert(usr <> nil);
  txtUsuario.Text := node.Text;
  txtNombreCompleto.Text := usr.FullName;
  txtDescripcion.Text := usr.Description;
  //txtEmail.Text := usr.GetEx('EmailAddress');
  flags := usr.Get('userFlags');
  opCambioContraProx.Checked := usr.Get('PasswordExpired');
  opNoCambioContra.Checked := (flags AND ADS_UF_PASSWD_CANT_CHANGE)<>0;
  opContraExpira.Checked := (flags and ADS_UF_DONT_EXPIRE_PASSWD)<>0;
  opCuentaDesactivada.Checked := usr.AccountDisabled;
  opCuentaBloqueada.Checked := usr.IsAccountLocked;
  EnumGroups(usr);
end;

procedure TformDatos.bExportarTodosClick(Sender: TObject);
var
  usr : IAdsUser;
  flags : integer;
  dominio : string;
  i, j : integer;
  fichero : TStringList;
  nombreUsuario, usuNombreCompleto, usuDescripcion, usuEmail : string;
  nombreFicheroDestino, usuGrupoSeguridad : string;
  usuCambioContraProx, usuNoCambioContra, usuContraExpira,
      usuCuentaDesactivada, usuCuentaBloqueada : boolean;
begin
  if MessageDlg('Se exportarn los usuarios del dominio y sus datos a un ' +
      'fichero de texto con los campos separados por Tabulador. ' +
      'De esta forma se podr abrir con Excel o cualquier otro programa ' +
      'que permita visualizar documentos separados con Tabulador. ' +
      'Desea continuar?', mtConfirmation, [mbyes, mbno], 0) = mryes then
  begin
    if dlGuardar.Execute then
    begin
      nombreFicheroDestino := dlGuardar.filename;
      fichero := TStringList.Create;
      fichero.Add('NOMBRE' + chr(9) + 'NOMBRE COMPLETO' + chr(9) +
          'DESCRIPCIN' + chr(9) + 'CAMBIO CONTRASEA PRXIMO INICIO SESIN' +
          chr(9) + 'CONTRASEA EXPIRA' + chr(9) + 'NO PUEDE CAMBIAR LA CONTRASEA' +
          chr(9) + 'DESACTIVADA' + chr(9) + 'BLOQUEADA' + chr(9)
          + 'GRUPOS SEGURIDAD');
      if formMenuPrincipal.opDominio.Checked then
        dominio := formMenuPrincipal.txtDominio.Text
      else
        dominio := formMenuPrincipal.txtEquipo.Text;
      for i := 0 to lsUsuarios.Items.Count - 1 do
      begin
        nombreUsuario := lsUsuarios.Items[i].Text;
        if formMenuPrincipal.opConectarUsuario.Checked then
          ADsOpenObject('WinNT://' + dominio + '/' +
              nombreUsuario, formMenuPrincipal.txtUsuario.Text,
              formMenuPrincipal.txtContrasena.Text, 1,
              IADsUser, usr)
        else
          ADsGetObject('WinNT://' + dominio + '/' + nombreUsuario, IADsUser, usr);

        Assert(usr <> nil);
        usuNombreCompleto := usr.FullName;
        usuDescripcion := usr.Description;
       // usuEmail := usr.EmailAddress;
        flags := usr.Get('userFlags');
        usuCambioContraProx := usr.Get('PasswordExpired');
        usuNoCambioContra := (flags AND ADS_UF_PASSWD_CANT_CHANGE) <> 0;
        usuContraExpira := (flags and ADS_UF_DONT_EXPIRE_PASSWD) <> 0;
        usuCuentaDesactivada := usr.AccountDisabled;
        usuCuentaBloqueada := usr.IsAccountLocked;

        EnumGroups(usr);

        usuGrupoSeguridad := '';
        for j := 0 to lbGroups.Items.Count - 1 do
        begin
          if usuGrupoSeguridad = '' then
            usuGrupoSeguridad := lbGroups.Items[j]
          else
            usuGrupoSeguridad := usuGrupoSeguridad + ';' + lbGroups.Items[j];
        end;

        fichero.Add(nombreUsuario + chr(9) + usuNombreCompleto +
             chr(9) + usuDescripcion + chr(9) + siNo (usuCambioContraProx) +
             chr(9) + siNo (usuContraExpira) + chr(9) +
             siNo (usuNoCambioContra) + chr(9) +
             siNo (usuCuentaDesactivada) + chr(9) +
             siNo (usuCuentaBloqueada) + chr(9) + usuEmail + chr(9) +
             usuGrupoSeguridad);
      end;
      fichero.SaveToFile(nombreFicheroDestino);
    end;
  end;
end;

end.
